/*
 * 
 * 
 */

package	com.idca.fsim.core;

import com.idca.fsim.core.jdbc.ConnectionFactory;
import	com.idca.fsim.core.plugins.SearchSource;


import	java.util.ArrayList;
import	java.util.HashMap;
import	java.util.Map;
import	java.util.List;
import	java.util.Iterator;
import	java.util.Collection;
import	java.util.EventListener;
import	java.util.logging.Logger;

import	java.sql.Connection;
import	java.sql.SQLException;


/**
 * An FsimContext
 * 
 * Note: because we use here only the non-synchronized Collections 
 * we must synchronize this objectes are stuctural changed (add/remove) !!
 * 
 * @author Willem Cazander
 * @version 1.0 
 */
public class FsimContext
{
	/** */
	private Logger logger = Logger.getLogger(FsimContext.class.getName());
	private Map searchSources = new HashMap();
	private List listeners = new ArrayList(1);
	private String name = null;
	
	private Map jspIncludePages = new HashMap(3);	

	// ============ jspIncludePages
	
	public void addJSPIncludePage(String name,String file)
	{
		synchronized(jspIncludePages)
		{
			jspIncludePages.put(name,file);
		}
		logger.info("Adding plugin page: "+name+" file: "+file);
	}
	public String getJSPIncludePage(String name)
	{
		return (String)jspIncludePages.get(name);
	}
	
	// ============== the name of this search context.
	
	public void setName(String name)
	{
		this.name = name;
	}
	public String getName()
	{
		return name;
	}
	
	
	// ================ listeners
	
	public List getListeners()
	{
		return listeners;
	}
	
	public void addListener(EventListener eventListener)
	{
		logger.info("Adding EventListener");
		synchronized(listeners)
		{
			listeners.add(eventListener);
		}
	}
	
	
	// ================= search sources
	
	public Collection getSearchSources()
	{
		return searchSources.values();
	}
	public SearchSource getSearchSource(String name)
	{
		return (SearchSource)searchSources.get(name);
	}
	public void addSearchSource(SearchSource searchSource)
	{
		logger.info("Adding dataSearchSource: "+searchSource.getName());
		synchronized(searchSources)
		{
			searchSources.put(searchSource.getName(),searchSource);
		}
	}
	
	
	// ================== the db functions.

	public Connection getConnectionByName(String dbName) throws SQLException
	{
		return ConnectionFactory.getConnection(dbName);
	}
}
