/*
 * Created on jul 16, 2005
 *
 * Copyright 2004 IDCA. All rights reserved.
 * IDCA PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package	com.idca.fsim.core;

import com.idca.fsim.core.config.ConfigParser;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import java.io.InputStream;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
 * Starts fsim from a web context.
 * @author Willem Cazander
 * @version 1.0 28/06/2005
 */
public class FsimListener implements ServletContextListener 
{
	/** */
	private Logger logger = Logger.getLogger(FsimListener.class.getName());
	
	/**
	 * This method is called when the servlet context is<br>
	 * initialized(when the Web application is deployed). <br>
	 * You can initialize servlet context related data here.<br>
	 */
	public void contextInitialized(ServletContextEvent event)
	{
		initLogging();
		
		ServletContext sc = event.getServletContext();
		String file = sc.getRealPath(sc.getInitParameter("FSIM_CONFIG_FILE"));
		if(file==null)
		{
			logger.warning("No config file given!! giving up, set parameter: "+"FSIM_CONFIG_FILE");
		//	return;
		}
		try {
			startFsim(file);
			
		} catch (Exception e) {
			logger.warning("Fsim could not be started: "+e.getMessage());
			e.printStackTrace();
		}
	}
	
	private void initLogging()
	{
		
    	try {
    		InputStream loggingProperties = null;
    		String dev = "false"; //ApplicationSettings.getSetting("DEVELOPMENT");
    		
    		if("true".equalsIgnoreCase(dev))
    		{
        		// 1st try to load the non-cvs properties
        		try
        		{
        			loggingProperties = this.getClass().getResourceAsStream("/com/idca/fsim/resources/logging/logging-development.properties");
        		} catch (Throwable overrideT) {
        		}
    		}	
    		// else the default properties
    		if(loggingProperties==null)
    		{
    			loggingProperties = this.getClass().getResourceAsStream("/com/idca/fsim/resources/logging/logging.properties");
    			logger.info("Loaded default logging properties");
    		} else {
    			logger.info("Loaded development logging properties");
    		}
    		
    		LogManager.getLogManager().readConfiguration( loggingProperties );
    		loggingProperties.close();
    	} catch (Throwable logT) {
    		// PRINT AS STACKTRACE because when we fuck the logging there is no log message 
    		logT.printStackTrace();
    	}
    	//reinit logger for new levels
    	 logger = Logger.getLogger(this.getClass().getName());
    	
    	checkLogLevels();
	}
	
	private void checkLogLevels()
	{
    	// TEST all log levels
    	logger.info			("TESTING LOG LEVELS:\n");
    	logger.finest		("LOGLEVEL ENABLED");
    	logger.finer		("LOGLEVEL ENABLED");
    	logger.fine			("LOGLEVEL ENABLED");
    	logger.config		("LOGLEVEL ENABLED");
    	logger.warning		("LOGLEVEL ENABLED");
    	logger.info			("LOGLEVEL ENABLED\n");		
	}
		
	
	
	
	/**
	 * This method is invoked when the Servlet Context.<br> 
	 * (the Web application) is undeployed or <br>
	 * the J2EE Server shuts down.<br>
	 */
	public void contextDestroyed(ServletContextEvent event)
	{
	}
	
	
	private void startFsim(String fileName)
	{
		try {
			FsimContext fc = ConfigParser.parse(fileName);
			FsimServer.addFsimContext(fc);
		} catch (Exception e) {
			logger.warning("Error while loading: "+fileName+" message: "+e.getMessage());
			e.printStackTrace();
		}
		
		logger.info("done starting fsim");	
	}
	

	
}
