/*
 * Created on jul 16, 2005
 *
 * Copyright 2004 IDCA. All rights reserved.
 * IDCA PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package	com.idca.fsim.core;

import com.idca.fsim.core.comparators.WordListComparator;
import	com.idca.fsim.models.Word;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Iterator;
import	java.util.Map;
import	java.util.Hashtable;
import	java.util.List;
import	java.util.HashSet;
import	java.util.Collection;
import	java.util.logging.Logger;

/**
 * An Words list
 * 
 * @author Willem Cazander
 * @version 1.0 
 */
public class WordLists
{	
	private Map letters = new Hashtable(140);
	private Logger logger = Logger.getLogger(WordLists.class.getName());
	
	/**
	 * 
	 * @param word
	 */
	public void put(Word word)
	{
		int wordLength = word.text.length();
		for(int i=0;i<wordLength-1;i++)
		{
			Character ch1 = new Character(word.text.charAt(i));
			Character ch2 = new Character(word.text.charAt(i+1));
			
			LetterMap letterMap = (LetterMap)letters.get(ch1);
			if(letterMap==null) {
				letterMap = new LetterMap();
				synchronized(letters)
				{
					letters.put(ch1,letterMap);
				}
			}
			synchronized(letterMap)
			{
				letterMap.addWord(ch2,word);
			}
		}
	}
	
	/**
	 * THis methode returns a Collection of all matching text
	 * like: 'text' => te+ex+xt = returned. 
	 * @param text
	 * @return
	 */
	public Collection getWords(String text)
	{
		//logger.info("Starting search on "+text);
		// no spaces should be in text
		int textLength = text.length();
		List allWords = new ArrayList(6);
		for(int i=0;i<textLength-1;i++)
		{
			Character ch1 = new Character(text.charAt(i));			
			Character ch2 = new Character(text.charAt(i+1));
			
			LetterMap letterMap = (LetterMap)letters.get(ch1);
			Collection words = null;
			if(letterMap!=null) {
				words = letterMap.getWords(ch2);
			}
			
			if(words==null) {
				//System.err.println("letterpair could not be found: '"+ch1+ch2+"'");
				continue;
			}
			// filter out double letterpairs 
			if(!allWords.contains(words)) {
				allWords.add(words);
			}
		}
		// sort on list size
		Collections.sort(allWords,new WordListComparator());
		
		Collection result = null;
		Iterator i = allWords.iterator();
		while(i.hasNext())
		{
			Collection list = (Collection)i.next();
			if(result==null) {
				//System.out.println("Starting with=null list="+list.size());
				result = new HashSet(list);
			} else {
				//System.out.println("retrain with="+result.size()+" list="+list.size());
				result.retainAll(list);
			}
			if(result.size()==0) {
				// not contineu if we didn't found anything.
				break;
			}
		}
		if(result==null) {
			result = new ArrayList(0);
		}
		
		//logger.info("returning result size: "+result.size());
		return result;
	}
	/**
	 * 
	 * @param text should not contain spaces !!
	 * @param inWord if false word should startWith text
	 * @param matchCase upper/lower case matching
	 * @param exactMatch then text equals word.text
	 * @return
	 */
	public List getWords(String text,boolean inWord,boolean matchCase,boolean exactMatch)
	{
		List result = new ArrayList(10);
		Collection words = getWords(text);
		Iterator i = words.iterator();
		
		if(!matchCase) {
			text = text.toLowerCase();
		}
		
		while(i.hasNext())
		{
			Word word = (Word)i.next();
			
			String wordText = word.text;
			if(!matchCase) {
				wordText = word.text.toLowerCase();
			}
			//System.out.println("=========search="+wordText+" list="+text);
			
			// think twice this is reversed code !!
			boolean found = true;
			int index = wordText.indexOf(text);
			if(!inWord) {
				// if is other the start pos them next word/
				if(index!=0) {
					found = false;
				}
			} else {
				if(index==-1) {
					found = false;
				}
			}
			if(found && exactMatch && !wordText.equals(text)) {
				found = false;
			}
			
			if(found) {
				result.add(word);
			}
		}
		return result;
	}
	
	
	public Word getWord(String text)
	{
		List col = getWords(text,false,true,true);
		if(col.size()==1) {
			return (Word)col.get(0);
		}
		if(col.size()==0) {
			// not found word.
			return null;
		}
		logger.warning("list is bigger than one should not happen: collection size: "+col.size()+" word="+text);
		
		Iterator i = col.iterator();
		while(i.hasNext()) {
			
			Word word = (Word)i.next();
			System.out.println(" word == "+word.text);
			
		}
		
		return null;
	}
	
	public LetterMap getLetterMap(Character ch)
	{
		return (LetterMap)letters.get(ch);
	}	
	
	public String dumpInfo()
	{
		Iterator keys = letters.keySet().iterator();
		while(keys.hasNext())
		{
			Object key1 = keys.next();
			LetterMap letterMap =(LetterMap)letters.get(key1);
			
			Iterator letters2 = letterMap.getWordSet().keySet().iterator();
			while(letters2.hasNext())
			{
				Object key2 = letters2.next();
				Collection list2 = (Collection)letterMap.getWordSet().get(key2);
				System.out.println("Letter: "+key1+key2+" words: "+list2.size());
			}
		}
		return null;
	}
}
