
package com.idca.fsim.core.indexer;

import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

/**
 * Manages the indexing
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 14, 2006
 */
public class IndexerManagerListener  implements ServletContextListener {
	
	/** The logger to log to. */
	private Logger logger = Logger.getLogger(IndexerManagerListener.class.getName());
	
	/**
	 *  The name of the initparameter where to register the jmx bean to.
	 * Set to "" or "null" to no register bean.
	 * */
	static public final String INDEXER_MANANGER_OBJECT_NAME = "IndexerManager.OBJECT_NAME";
	
	static private IndexerManager indexerManager = null;
	
	/**
	 * Starts Hibernate
	 */
	public void contextInitialized(ServletContextEvent event) {
		logger.info("Creating the IndexerManager.");
		try {
			indexerManager = new IndexerManager();
			String objectName=event.getServletContext().getInitParameter(INDEXER_MANANGER_OBJECT_NAME);
			if(objectName==null)
			{
				//indexerManager.setObjectName("M4N:type=datafeeds,name=ParserThreadPoolManager");
				//indexerManager.register();
			} else {
				if("".equals(objectName) | "null".equals(objectName))
				{
					logger.info("Not registrating IndexerManager by user command");
				} else {
					//indexerManager.setObjectName(objectName);
					//indexerManager.register();
				}
			}		
		} catch (Exception e) {
			logger.log(Level.SEVERE,"Error while configing IndexerManager.",e);
		}
	}
	
	/**
	 * Shutdowns Hibernate
	 */
	public void contextDestroyed(ServletContextEvent event) {
		try {
			logger.info("Shutdowing the IndexerManager.");
			indexerManager.getThreadPoolExecutor().shutdown();
		} catch (Exception e) {
			logger.warning("Error while shutdowning the IndexerManager");
		}
	}
	
	/**
	 * Returns the IndexManager
	 * @return
	 */
	static public IndexerManager getIndexerManager() {
		return indexerManager;
	}
}