/*
 * Created on jul 16, 2005
 *
 * Copyright 2004 IDCA. All rights reserved.
 * IDCA PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package com.idca.fsim.web;

import	com.idca.fsim.core.*;
import	com.idca.fsim.core.plugins.*;
import	com.idca.fsim.models.*;
import	com.idca.fsim.web.session.SearchBean;

import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import	javax.servlet.RequestDispatcher;

import	com.idca.fsim.searchers.files.collectors.*;

import	java.util.*;
import	java.util.logging.Logger;

/**
 * The contoll servlet for the search pages
 * 
 * @author Willem Cazander
 * @version 1.0 
 */
public class SearchServlet extends HttpServlet
{
	/** */
	private FsimContext fsimContext = null;
	/** */
	private Logger logger = Logger.getLogger(SearchServlet.class.getName());
	
	
	/**
	 * 
	 */
	public void init(ServletConfig servletConfig) throws ServletException
	{    
		fsimContext = FsimServer.getFsimContext(servletConfig.getServletContext().getInitParameter("FsimContextName"));
		if(fsimContext==null) {
			throw new ServletException("could not find fsimContextName: "+servletConfig.getServletContext().getInitParameter("FsimContextName"));
		}
	}
	
	
	/**
	 * The service method of the servlet. <br>
	 *
	 * This method is called when a form has its tag value method equals to get.
	 * 
	 * @param request the request send by the client to the server
	 * @param response the response send by the server to the client
	 * @throws ServletException if an error occurred
	 * @throws IOException if an error occurred
	 */
	public void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException
	{
		try {
		    HttpServletRequest httpRequest = (HttpServletRequest)request;
			HttpServletResponse httpResponse = (HttpServletResponse)response;
			HttpSession session = httpRequest.getSession(false);
			
			response.setHeader("Cache-Control", "no-cache");
			PrintWriter out = response.getWriter();
			
			SearchBean searchBean = (SearchBean)session.getAttribute("searchbean");
			
			String mode = request.getParameter("m");
			int m = 0;
			if(mode!=null) {
				m = new Integer(mode).intValue();
			}
			
			String sources = (String)session.getAttribute("searchSources");
			if(sources==null) {
				sources = "fsim-files;fsim-paths;fsim-hosts";  // default search all
				session.setAttribute("searchSources",sources);
			} else {
				sources = "";
				if(request.getParameter("fsim-files")!=null) {
					sources = "fsim-files";
				}
				if(request.getParameter("fsim-paths")!=null) {
					sources += ";fsim-paths";
				}
				if(request.getParameter("fsim-hosts")!=null) {
					sources += ";fsim-hosts";
				}
				session.setAttribute("searchSources",sources);
			}
			
			
			
			switch(m)
			{
			
			
				default:
				case 0:
					// an normal full page search
					response.setContentType("text/html; charset=UTF-8");
					request.setAttribute("searchSources2",fsimContext.getSearchSources());
					long start = System.currentTimeMillis();
					
					int p = 0;
					String page = request.getParameter("p");
					if(page!=null)
					{
						try {
							p = new Integer(page).intValue();
						} catch (Exception e) {
							// block fukker.
							p = 0;
						}
					}
					searchBean.setSearchSources(sources);
					
					String q = request.getParameter("q");
					if(q==null) {
					    q = "";
					}
					if(q.length()>40) {
					    q = "max query size";
					}
					
					searchBean.setSearchQuery(q);	
					searchBean.doSearch();
					
					searchBean.setPageIndex(p);
					
					long searchTime = System.currentTimeMillis();

					request.setAttribute("searchBean",searchBean);
					request.setAttribute("resultSize",new Integer(searchBean.getSearchHits().size()));
					request.setAttribute("searchTime",new Long(searchTime-start));
					
					request.setAttribute("drawSearch",new Boolean(true));
					request.setAttribute("drawPageResults",new Boolean(true));
					
					RequestDispatcher top = request.getRequestDispatcher("fragments/fsim_top.jsp");
					top.include(request, response);		
					
					Collection pageResults = searchBean.getModelHitTublePageResults();
					if(pageResults.size()>0) 
					{	
						Iterator i = pageResults.iterator();
						while(i.hasNext()) {
							ModelHitTuble modelHit = (ModelHitTuble)i.next();
							//System.out.println("source="+modelHit.getHit().source.getName()+" model="+modelHit.getModel().getClass().getName());
							modelHit.getHit().source.getResultRenderer().buildHTML(out,modelHit.getHit(),modelHit.getModel(),searchBean.getSearchQuery());
						}
					}
					long renderTime = System.currentTimeMillis();
					request.setAttribute("renderTime",new Long(renderTime-searchTime));
					RequestDispatcher bottom = request.getRequestDispatcher("fragments/fsim_bottom.jsp");
					bottom.include(request, response);
					
					break;
				case 1:
					// THE XML keywords
					//System.out.println("mode 1 building xml list");
					response.setContentType("text/xml; charset=UTF-8");
					
					String letters = request.getParameter("l");
					// small hack :)
					String[] wd = letters.split(" ");
					letters = wd[wd.length-1];
					if(letters!=null) {
						
						Searcher searcher = new Searcher(fsimContext.getSearchSource("fsim-files").getWordLists());
						searcher.setSearchSources(sources);
						Collection result = searcher.searchWords(letters,false);
						
						Iterator i = result.iterator();
						int max = 20;
						out.println("<items>");
						while(i.hasNext()) {
							Word word = (Word)i.next();
							out.println("<item hits=\""+word.hits.size()+"\" clicked=\""+word.clicked+"\"><![CDATA["+word.text+"]]></item>");
							max--;
							if(max==0) {
								break;
							}
						}
						
						out.println("</items>");
						
					}
					
					break;
			}
			
			//out.flush();
		//	out.close();
		} catch (Exception e) {
			e.printStackTrace();
			
		}
	}

	
	public long getLastModified(HttpServletRequest request) {
		return System.currentTimeMillis();
	}	

	/**
	 * Returns information about the servlet, such as 
	 * author, version, and copyright. 
	 *
	 * @return String information about this servlet
	 */
	public String getServletInfo() {
		return "The FSIM search servlet (c)2005 IDCA";
	}

	/**
	 * Destruction of the servlet. <br>
	 */
	public void destroy() {
		super.destroy(); // Just puts "destroy" string in log
	}
	
}
